/*
  *  Copyright © 2014 Copyright Clearance Center, Inc.
  *
  * Copyright Clearance Center licenses this file to You under the Apache License,
  * Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software distributed under
 *   the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *   either express or implied. See the License for the specific language governing permissions and
 *   limitations under the License.
*/
package com.copyright.developers.blog.rest;

import com.copyright.developers.blog.domain.SampleDomainObject;
import com.copyright.developers.blog.validators.IdValidator;
import com.copyright.developers.blog.validators.NameValidator;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.springframework.stereotype.Controller;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import java.io.IOException;
import java.util.Arrays;
import javax.servlet.http.HttpServletResponse;

@Controller
@RequestMapping("/sample")
public class SampleController {

    /**
     * The validators specified on the parameters of this method other than one annotated with
     * <code>@RequestBody</code> will not be executed.
     */
    @RequestMapping(value = "/test_1/{id}", method = RequestMethod.POST)
    public void methodOneWithParams(@Validated(IdValidator.class) @PathVariable String id,
            @Validated(NameValidator.class) @RequestParam String name,
            @Validated @RequestBody SampleDomainObject sampleDomainObject) {

        System.out.println("Executing method 1");
        System.out.println("ID: " + id);
        System.out.println("NAME: " + name);
        System.out.println("OBJECT: " + ToStringBuilder.reflectionToString(sampleDomainObject));
    }

    /**
     * All validators will be executed.
     */
    @RequestMapping(value = "/test_2/{id}", method = RequestMethod.POST)
    public void methodTwoWithParams(
            @ModelAttribute("id") @Validated(IdValidator.class) String id,
            @ModelAttribute("name") @Validated(NameValidator.class) String name,
            @Validated @RequestBody SampleDomainObject sampleDomainObject) {

        System.out.println("Executing method 2");
        System.out.println("ID: " + id);
        System.out.println("NAME: " + name);
        System.out.println("OBJECT: " + ToStringBuilder.reflectionToString(sampleDomainObject));
    }

    /**
     * This exception is produced in case of validation errors encountered while validating
     * parameters annotated with <code>@RequestBody</code>.
     */
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public void onValidationException(MethodArgumentNotValidException e,
            HttpServletResponse response) throws IOException {

        StringBuilder errorMessages = getErrorMessages(e.getBindingResult());
        response.sendError(HttpServletResponse.SC_BAD_REQUEST, errorMessages.toString());
    }

    /**
     * This exception is produced in case of validation errors encountered while validating
     * parameters annotated with <code>@ModelAttribute</code>.
     */
    @ExceptionHandler(BindException.class)
    public void onValidationException(BindException e,
            HttpServletResponse response) throws IOException {

        StringBuilder errorMessages = getErrorMessages(e.getBindingResult());
        response.sendError(HttpServletResponse.SC_BAD_REQUEST, errorMessages.toString());
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////////////////////////

    @ModelAttribute("id")
    public String idAsModelAttribute(@PathVariable String id) {
        return id;
    }

    @ModelAttribute("name")
    public String nameAsModelAttribute(@RequestParam String name) {
        return name;
    }

    private StringBuilder getErrorMessages(BindingResult results) {

        StringBuilder errorMessages = new StringBuilder();
        for(ObjectError error : results.getAllErrors()) {
            errorMessages.append(Arrays.toString(error.getCodes())).append("\n");
        }

        return errorMessages;
    }
}
