/*
  *  Copyright © 2014 Copyright Clearance Center, Inc.
  *
  * Copyright Clearance Center licenses this file to You under the Apache License,
  * Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software distributed under
 *   the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 *   either express or implied. See the License for the specific language governing permissions and
 *   limitations under the License.
*/
package com.copyright.developers.blog.rest;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

import com.copyright.developers.blog.domain.SampleDomainObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.context.web.WebAppConfiguration;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;
import org.springframework.web.context.WebApplicationContext;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration("classpath:sample-rest-context.xml")
@WebAppConfiguration
public class SampleControllerTest {

    private static final String URI_TEMPLATE_FOR_TEST_1 = "/sample/test_1/{id}?name={name}";
    private static final String URI_TEMPLATE_FOR_TEST_2 = "/sample/test_2/{id}?name={name}";

    private static final SampleDomainObject VALID_SAMPLE_DOMAIN_OBJECT =
            new SampleDomainObject("1", "name", "desc");
    private static final SampleDomainObject INVALID_SAMPLE_DOMAIN_OBJECT =
            new SampleDomainObject(null, "name", "desc");

    @Autowired
    private WebApplicationContext webApplicationContext;

    private ObjectMapper objectMapper = new ObjectMapper();
    private MockMvc mockMvc;

    @Before
    public void setUp() {
        mockMvc = MockMvcBuilders.webAppContextSetup(webApplicationContext).build();
    }

    @Test
    public void testMethodOneSuccess() throws Exception {

        mockMvc.perform(
                post(URI_TEMPLATE_FOR_TEST_1, "ABC123", "THREE")
                        .content(objectMapper.writeValueAsString(VALID_SAMPLE_DOMAIN_OBJECT))
                        .contentType(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isOk());
    }

    //    @Test
    @Ignore("If executed this test case will fail since validators are not called as expected")
    public void testMethodOneFailBadPathParameter() throws Exception {

        mockMvc.perform(
                post(URI_TEMPLATE_FOR_TEST_1, "ABC-123", "THREE")
                        .content(objectMapper.writeValueAsString(VALID_SAMPLE_DOMAIN_OBJECT))
                        .contentType(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isBadRequest());
    }

    //    @Test
    @Ignore("If executed this test case will fail since validators are not called as expected")
    public void testMethodOneFailBadRequestParameter() throws Exception {

        mockMvc.perform(
                post(URI_TEMPLATE_FOR_TEST_1, "ABC123", "three")
                        .content(objectMapper.writeValueAsString(VALID_SAMPLE_DOMAIN_OBJECT))
                        .contentType(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isBadRequest());
    }

    @Test
    public void testMethodOneFailBadPayload() throws Exception {

        mockMvc.perform(
                post(URI_TEMPLATE_FOR_TEST_1, "ABC123", "three")
                        .content(objectMapper.writeValueAsString(INVALID_SAMPLE_DOMAIN_OBJECT))
                        .contentType(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isBadRequest());
    }

    @Test
    public void testMethodTwoSuccess() throws Exception {

        mockMvc.perform(
                post(URI_TEMPLATE_FOR_TEST_2, "ABC123", "THREE")
                        .content(objectMapper.writeValueAsString(VALID_SAMPLE_DOMAIN_OBJECT))
                        .contentType(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isOk());
    }

    @Test
    public void testMethodTwoFailBadPathParameter() throws Exception {

        mockMvc.perform(
                post(URI_TEMPLATE_FOR_TEST_2, "ABC-123", "THREE")
                        .content(objectMapper.writeValueAsString(VALID_SAMPLE_DOMAIN_OBJECT))
                        .contentType(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isBadRequest());
    }

    @Test
    public void testMethodTwoFailBadRequestParameter() throws Exception {

        mockMvc.perform(
                post(URI_TEMPLATE_FOR_TEST_2, "ABC123", "three")
                        .content(objectMapper.writeValueAsString(VALID_SAMPLE_DOMAIN_OBJECT))
                        .contentType(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isBadRequest());
    }

    @Test
    public void testMethodTwoFailBadPayload() throws Exception {

        mockMvc.perform(
                post(URI_TEMPLATE_FOR_TEST_2, "ABC123", "three")
                        .content(objectMapper.writeValueAsString(INVALID_SAMPLE_DOMAIN_OBJECT))
                        .contentType(MediaType.APPLICATION_JSON))
                .andDo(print())
                .andExpect(status().isBadRequest());
    }
}
